#!/usr/bin/env python

from .resource import Resource
from .daily_activity_transaction import DailyActivityTransaction


class DailyActivity(Resource):
    """This resource allows partners to access their users' daily activity data.

    https://www.polar.com/accesslink-api/?http#daily-activity
    """
    def create_transaction(self, user_id, access_token):
        """Initiate daily activity transaction

        Check for new daily activity and create a new transaction if data is available.

        :param user_id: id of the user
        :param access_token: access token of the user
        """
        response = self._post(endpoint="/users/{}/activity-transactions".format(user_id),
                              access_token=access_token)
        if not response:
            return None

        return DailyActivityTransaction(oauth=self.oauth,
                                        transaction_url=response["resource-uri"],
                                        user_id=user_id,
                                        access_token=access_token)


#!/usr/bin/env python

from .transaction import Transaction


class TrainingDataTransaction(Transaction):

    def list_exercises(self):
        """Retrieve list of urls to available exercises

        After successfully initiating a transaction, training sessions included
        within it can be retrieved with the provided transactionId.
        """
        return self._get(endpoint=None, url=self.transaction_url,
                         access_token=self.access_token)

    def get_exercise_summary(self, url):
        """Retrieve training session summary data

        :param url: url of the exercise entity
        """
        return self._get(endpoint=None, url=url,
                         access_token=self.access_token)

    def get_gpx(self, url):
        """Retrieve training session summary data in GPX format

        :param url: url of the exercise entity
        """
        return self._get(endpoint=None, url=url+"/gpx",
                         access_token=self.access_token,
                         headers={"Accept": "application/gpx+xml"})

    def get_tcx(self, url):
        """Retrieve training session summary data in TCX format

        :param url: url of the exercise entity
        """
        return self._get(endpoint=None, url=url+"/tcx",
                         access_token=self.access_token,
                         headers={"Accept": "application/vnd.garmin.tcx+xml"})

    def get_heart_rate_zones(self, url):
        """Retrieve heart rate zones in training session

        :param url: url of the exercise entity
        """
        return self._get(endpoint=None, url=url+"/heart-rate-zones",
                         access_token=self.access_token)

    def get_available_samples(self, url):
        """Retrieve list of urls to available samples in training session

        :param url: url of the exercise entity
        """
        return self._get(endpoint=None, url=url+"/samples",
                         access_token=self.access_token)

    def get_samples(self, url):
        """Retrieve sample data of given type

        :param url: url pointing to single sample type data
        """
        return self._get(endpoint=None, url=url,
                         access_token=self.access_token)





#!/usr/bin/env python

from .transaction import Transaction


class PhysicalInfoTransaction(Transaction):

    def list_physical_infos(self):
        """Get a list of physical info resource urls in the transaction"""
        return self._get(endpoint=None, url=self.transaction_url,
                         access_token=self.access_token)

    def get_physical_info(self, url):
        """Get user's physical information from the transaction

        :param url: url of the physical info entity
        """
        return self._get(endpoint=None, url=url,
                         access_token=self.access_token)


#!/usr/bin/env python

import uuid

from .resource import Resource


class Users(Resource):
    """This resource provides all the necessary functions to manage users.

    https://www.polar.com/accesslink-api/?http#users
    """

    def register(self, access_token, member_id=uuid.uuid4().hex):
        """Registration

        Once partner has been authorized by user, partner must register user before being able to access her data.

        :param access_token: access token of the user
        :param member_id: unique client-specific identifier for the user
        """
        return self._post(endpoint="/users",
                          access_token=access_token,
                          json={"member-id": member_id})

    def delete(self, user_id, access_token):
        """De-registration

        When partner wishes no longer to receive user data, user can be de-registered.
        This will revoke the access token authorized by user.

        :param user_id: id of the user
        :param access_token: access token of the user
        """
        return self._delete(endpoint="/users/{user_id}".format(user_id=user_id),
                            access_token=access_token)

    def get_information(self, user_id, access_token):
        """List user's basic information.

        :param user_id: id of the user
        :param access_token: access token of the user
        """
        return self._get(endpoint="/users/{user_id}".format(user_id=user_id),
                         access_token=access_token)



#!/usr/bin/env python

from .resource import Resource


class Transaction(Resource):

    def __init__(self, oauth, transaction_url, user_id, access_token):
        super(Transaction, self).__init__(oauth)
        self.transaction_url = transaction_url
        self.user_id = user_id
        self.access_token = access_token

    def commit(self):
        """Commit the transaction

        This should be done after retrieving data from the transaction.
        """
        return self._put(endpoint=None, url=self.transaction_url,
                         access_token=self.access_token)



#!/usr/bin/env python


class Resource(object):

    def __init__(self, oauth):
        self.oauth = oauth

    def _get(self, *args, **kwargs):
        return self.oauth.get(*args, **kwargs)

    def _post(self, *args, **kwargs):
        return self.oauth.post(*args, **kwargs)

    def _put(self, *args, **kwargs):
        return self.oauth.put(*args, **kwargs)

    def _delete(self, *args, **kwargs):
        return self.oauth.delete(*args, **kwargs)


#!/usr/bin/env python

import logging

import requests
from requests.auth import HTTPBasicAuth
from requests.exceptions import HTTPError

try:
    from urllib.parse import urlencode
except ImportError:
    from urllib import urlencode

_LOGGER = logging.getLogger(__name__)

class OAuth2Client(object):
    """Wrapper class for OAuth2 requests"""

    def __init__(self, url, authorization_url, access_token_url, redirect_url,
                 client_id, client_secret):
        self.url = url
        self.authorization_url = authorization_url
        self.access_token_url = access_token_url
        self.redirect_url = redirect_url
        self.client_id = client_id
        self.client_secret = client_secret

    def get_auth_headers(self, access_token):
        """Get authorization headers for user level api resources"""

        return {
            "Authorization": "Bearer {}".format(access_token),
            "Content-Type": "application/json",
            "Accept": "application/json"
        }

    def get_authorization_url(self, response_type="code", state=None):
        """Build authorization url for the client"""

        params = {
            "client_id": self.client_id,
            "response_type": response_type,
        }

        if state:
            params["state"] = state

        if self.redirect_url:
            params["redirect_uri"] = self.redirect_url

        return "{url}?{params}".format(url=self.authorization_url,
                                       params=urlencode(params))

    def get_access_token(self, authorization_code):
        """Exchange authorization code for an access token"""

        headers = {
            "Content-Type" : "application/x-www-form-urlencoded",
            "Accept" : "application/json;charset=UTF-8"
        }

        data = {
            "grant_type" : "authorization_code",
            "code" : authorization_code
        }

        if self.redirect_url:
            data["redirect_uri"] = self.redirect_url

        _LOGGER.debug('Fetching access token from auth code')

        return self.post(endpoint=None,
                         url=self.access_token_url,
                         data=data,
                         headers=headers)

    def __build_endpoint_kwargs(self, **kwargs):
        """Create endpoint url for requests

        If `endpoint` argument is given, it is appended to the api url
        and used as the request url. Otherwise `url` argument is used.
        """

        if "endpoint" in kwargs:
            if kwargs["endpoint"] is not None:
                kwargs["url"] = self.url + kwargs["endpoint"]
            del kwargs["endpoint"]

        return kwargs

    def __build_auth_kwargs(self, **kwargs):
        """Setup authentication for requests

        If `access_token` is given, it is used in Authentication header.
        Otherwise basic auth is used with the client credentials.
        """

        if "access_token" in kwargs:
            headers = self.get_auth_headers(kwargs["access_token"])

            if "headers" in kwargs:
                headers.update(kwargs["headers"])

            kwargs["headers"] = headers
            del kwargs["access_token"]
        elif "auth" not in kwargs:
            kwargs["auth"] = HTTPBasicAuth(self.client_id, self.client_secret)

        return kwargs

    def __build_request_kwargs(self, **kwargs):
        kwargs = self.__build_endpoint_kwargs(**kwargs)
        kwargs = self.__build_auth_kwargs(**kwargs)
        return kwargs

    def __parse_response(self, response):
        if response.status_code >= 400:
            message = "{code} {reason}: {body}".format(code=response.status_code,
                                                       reason=response.reason,
                                                       body=response.text)
            raise HTTPError(message, response=response)

        if response.status_code is 204:
            return {}

        try:
            return response.json()
        except ValueError:
            return response.text

    def __request(self, method, **kwargs):
        kwargs = self.__build_request_kwargs(**kwargs)

        _LOGGER.debug("%s request to URL: %s", method.upper(), kwargs["url"])

        response = requests.request(method, **kwargs)
        return self.__parse_response(response)

    def get(self, endpoint, **kwargs):
        return self.__request("get", endpoint=endpoint, **kwargs)

    def post(self, endpoint, **kwargs):
        return self.__request("post", endpoint=endpoint, **kwargs)

    def put(self, endpoint, **kwargs):
        return self.__request("put", endpoint=endpoint, **kwargs)

    def delete(self, endpoint, **kwargs):
        return self.__request("delete", endpoint=endpoint, **kwargs)



#!/usr/bin/env python

from .transaction import Transaction


class DailyActivityTransaction(Transaction):

    def list_activities(self):
        """Get a list of activity resource urls in the transaction"""
        return self._get(endpoint=None, url=self.transaction_url,
                         access_token=self.access_token)

    def get_activity_summary(self, url):
        """Get user's activity summary from the transaction

        :param url: url of the activity entity
        """
        return self._get(endpoint=None, url=url,
                         access_token=self.access_token)

    def get_step_samples(self, url):
        """Get activity step samples

        :param url: url of the activity entity
        """
        return self._get(endpoint=None, url=url+"/step-samples",
                         access_token=self.access_token)

    def get_zone_samples(self, url):
        """Get activity zone samples

        :param url: url of the activity entity
        """
        return self._get(endpoint=None, url=url+"/zone-samples",
                         access_token=self.access_token)



#!/usr/bin/env python

from . import endpoints
from .oauth2 import OAuth2Client

AUTHORIZATION_URL = "https://flow.polar.com/oauth2/authorization"
ACCESS_TOKEN_URL = "https://polarremote.com/v2/oauth2/token"
ACCESSLINK_URL = "https://www.polaraccesslink.com/v3"


class AccessLink(object):
    """Wrapper class for Polar Open AccessLink API v3"""

    def __init__(self, client_id, client_secret, redirect_url=None):
        if not client_id or not client_secret:
            raise ValueError("Client id and secret must be provided.")

        self.oauth = OAuth2Client(url=ACCESSLINK_URL,
                                  authorization_url=AUTHORIZATION_URL,
                                  access_token_url=ACCESS_TOKEN_URL,
                                  redirect_url=redirect_url,
                                  client_id=client_id,
                                  client_secret=client_secret)

        self.users = endpoints.Users(oauth=self.oauth)
        self.pull_notifications = endpoints.PullNotifications(oauth=self.oauth)
        self.training_data = endpoints.TrainingData(oauth=self.oauth)
        self.physical_info = endpoints.PhysicalInfo(oauth=self.oauth)
        self.daily_activity = endpoints.DailyActivity(oauth=self.oauth)

    def get_authorization_url(self, state=None):
        """Get the authorization url for the client"""
        return self.oauth.get_authorization_url(state=state)

    def get_access_token(self, authorization_code):
        """Request access token for a user.

        :param authorization_code: authorization code received from authorization endpoint.
        """
        return self.oauth.get_access_token(authorization_code)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand
import sys

with open('README.md') as readme_file:
    readme = readme_file.read()

requirements = [
    'requests'
]

setup_requirements = [ ]

test_requirements = [ ]

setup(
    version='0.0.5',
    author="Nat Burns",
    author_email='nbaccount@burnskids.com',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    description="Python wrapper for Polar Open AccessLink API.",
    install_requires=requirements,
    license="MIT license",
    long_description=readme,
    long_description_content_type="text/markdown",
    include_package_data=True,
    keywords='polar_accesslink',
    name='polar_accesslink',
    packages=find_packages(exclude=['tests']),
    setup_requires=setup_requirements,
    url='https://github.com/burnnat/polar_accesslink',
    zip_safe=False,
)



#!/usr/bin/env python

from .resource import Resource


class PullNotifications(Resource):
    """This resource allows partners to check if their users have available data for downloading

    https://www.polar.com/accesslink-api/?http#pull-notifications
    """

    def list(self):
        """List available data

        Get list of available exercises and activities for users
        """
        return self._get(endpoint="/notifications")
